==> pawm-1.9.7.tar.bz2: pawm-1.9.7/src/wm.c <==
/*
    Filename: wm.c
    Synopsis: Window management (general)

                          Copyright (C) 2000, 2001
            Ral Nez de Arenas Coronado <raul@pleyades.net>
	         David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to:

        Free Software Foundation, Inc.
        59 Temple Place, Suite 330
        Boston, MA 02111-1307 USA

*/

#include "x.h"
#include "wm.h"
#include "pabar.h"
#include "paclock.h"
#include "pimp.h"
#include "hints.h"
#include "pawindow.h"
#include "mobs.h"


/* Some definitions ********************************************************/
#define TOUGH  "Hey man, I'm the Puto Amo, and I want to nick my own sluts.\n"
#define POLITE "In other words, there is another Window Manager running...\n"
#define PAWM_EVMASK (SubstructureRedirectMask|ExposureMask)



/* Function prototypes *****************************************************/

/* Event handling */
static void wm_handle (XEvent *, void *);
static void wm_MapRequest       (XMapRequestEvent *);
static void wm_ConfigureRequest (XConfigureRequestEvent *);
static void wm_Expose		(XExposeEvent *);

/* Private helper functions */
static void wm_adoptwins (void);
static int wm_tmpeh (Display *, XErrorEvent *);
/****************************************************************************/

static volatile sig_atomic_t FuckingBoss=1;  /* We are the Puto Amo */
static unsigned short int reference_count=0;



/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/

static void wm_handle (XEvent *xevent, void *widget) {

    ASSERT(xevent);

    /* Widget is not need, just avoid the gcc warning */
    widget=NULL;

    switch (xevent->type) {
    case MapRequest:
        GOT(MapRequest, WM);
        wm_MapRequest(&(xevent->xmaprequest));
        break;
    case ConfigureRequest:
        GOT(ConfigureRequest, WM);
        wm_ConfigureRequest(&(xevent->xconfigurerequest));
        break;
    case Expose:
	GOT(Expose, WM);
	wm_Expose(&(xevent->xexpose));
	break;
    case ClientMessage:
        GOT(ClientMessage, WM);
        break;	
    default:
        SAY("Event %d on WM from window %#lx ", xevent->type, xevent->xunmap.window);

    }
}




static void wm_MapRequest (XMapRequestEvent *event) {

    PAWindow pawindow=NULL;
    XWMHints *wmHints;
    Window root=None;
    Window parent=None;
    Window *winstack=NULL;
    unsigned int howmany=0;

    ASSERT(event);
    
    if (!XQueryTree(display, event->window, &root,
                    &parent, &winstack, &howmany)) return;
    if (winstack) XFree(winstack);

    /*
        Here we must deal with a race-condition. While a window is being
    handled it can unmap itself and after a little while remap. So we can
    have a MapRequest even that event should be directed to the 'handler'
    window. We try to get it solved here. Please note that grabbing the
    server doesn't solve this problem, since the event is put in the queue
    before we even think about doing the grab.
    */
    if (parent!=ROOT) {
        XEvent e;

        e.type=MapRequest;
        e.xmaprequest.serial=NextRequest(display);
        e.xmaprequest.send_event=True;
        e.xmaprequest.display=display;
        e.xmaprequest.parent=parent;
        e.xmaprequest.window=event->window;
                
        if (!XSendEvent(display, parent, False, SubstructureRedirectMask, &e))
            BUG();
        return;
    }

    /* Manage the fucking window */
    pawindow=pawindow_create(event->window);
    
    /* Get the initial_state property, if set by the client */
    wmHints=XGetWMHints(display, event->window);
    if ((wmHints) && (wmHints->flags & StateHint) &&
        (wmHints->initial_state==IconicState)) {
	SAY("Client window wish to start iconified");
    }
    else {
	/* Client don't ask for IconicState..., map the fucking pawindow */
        //XGrabServer(display);
        pawindow_map(pawindow);
        //XUngrabServer(display);
    }
}




static void wm_ConfigureRequest (XConfigureRequestEvent *event) {

    XWindowChanges wc;
    Window root=None;
    Window parent=None;
    Window *winstack=NULL;
    unsigned int howmany=0;

    ASSERT(event);

    /*
        Here used to be a big BUG(), but since it faulted when a
    race condition arose, it was replaced by a shame return.
    X Clients really suck. GTK sucks. QT sucks, too. All sucks.
    */
    if (!XQueryTree(display, event->window, &root,
                    &parent, &winstack, &howmany)) return;
    if (winstack) XFree(winstack);

    if (parent!=ROOT) {
        XEvent e;
	
        e.type=ConfigureRequest;
        e.xconfigurerequest.serial=NextRequest(display);
        e.xconfigurerequest.send_event=True;
        e.xconfigurerequest.display=display;
        e.xconfigurerequest.parent=parent;
        e.xconfigurerequest.window=event->window;
        e.xconfigurerequest.x=event->x;
        e.xconfigurerequest.y=event->y;
        e.xconfigurerequest.width=event->width;
        e.xconfigurerequest.height=event->height;
        e.xconfigurerequest.border_width=event->border_width;
        e.xconfigurerequest.above=event->above;
        e.xconfigurerequest.detail=event->detail;
        e.xconfigurerequest.value_mask=event->value_mask;

        if (!XSendEvent(display, parent, False, SubstructureRedirectMask, &e))
            BUG();
        return;
    }

    /*
       As a guy told once upon a time: most of the fields
    contains crap, but there's no worry about that, since
    the X server (hopefully) won't take any members not
    specified in the value_mask. The Intel X server may
    be an exception, though. Intel...
    */
    wc.x=event->x;
    wc.y=event->y;
    wc.width=event->width;
    wc.height=event->height;
    wc.border_width=event->border_width;
    wc.sibling=event->above;
    wc.stack_mode=event->detail;

    XConfigureWindow(display, event->window, event->value_mask, &wc);
}


static void wm_Expose (XExposeEvent *event) {

    ASSERT(event);

    if (event->count) return;

    //x_draw_background();
    //pimp_mapall();

}


/****************************************************************************/





/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/

static int wm_tmpeh (Display *dpy, XErrorEvent *xerror) {

    ASSERT(dpy==display);

    dpy=NULL; xerror=NULL;

    /* Oh-oh... If we are here is because other WM is running */
    FuckingBoss=0;
    return 0;
}




/* This function adopts and manage all present windows on the desktop */
static void wm_adoptwins (void) {

    Window root=None;
    Window parent=None;
    Window *tree=NULL;
    unsigned int howmany=0;
    Status status;
    unsigned int i=0;

    /* Get info*/        
    status=XQueryTree(display, ROOT, &root, &parent, &tree, &howmany);

    /* Maybe no windows? */
    if ((!status) || (!howmany) || (!tree)) return;

    /* Try to manage all windows */
    for (i=0; i < howmany; i++) {
        XWindowAttributes attr;
        XEvent e;

        if(!XGetWindowAttributes(display, tree[i], &attr)) continue;
        /* Nothing to do if the window is not viewable */
        if (attr.map_state!=IsViewable) continue;
        if (attr.override_redirect==True) continue;

        XUnmapWindow(display, tree[i]);
        e.type=MapRequest;
        e.xmaprequest.serial=NextRequest(display);
        e.xmaprequest.send_event=False;
        e.xmaprequest.display=display;
        e.xmaprequest.parent=ROOT;
        e.xmaprequest.window=tree[i];

        /* Eat this... */
        if (!XSendEvent(display, ROOT, False, SubstructureRedirectMask, &e))
            BUG();
    }

    /* Be polite... */
    XFree(tree);
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
int wm_init (void) {

    char *tmpvalue;

    SAY ("Initializing window manager...");
    int (*olderrorhandler) (Display *, XErrorEvent *);

    if (reference_count++) return !0;
 
    olderrorhandler=XSetErrorHandler(wm_tmpeh);
    XSelectInput(display, ROOT, PAWM_EVMASK);
    XSync(display, False);
    XSetErrorHandler(olderrorhandler);

    if (!FuckingBoss) {
        fprintf(stderr, TOUGH);
        fprintf(stderr, POLITE);
        return 0;
    }

    /* Create a pretty background, using a background color */
    XSetWindowBackground(display, ROOT, x_color("background"));
    XClearWindow(display,ROOT);
    
    /* Initialize subsystems */
    SAY ("Initializing pawindows...");
    pawindow_init();

    /* Register the event handler ('sloot') */
    x_addsloot(ROOT, wm_handle, NULL);  /* No widget here */

    /* Signal API users that we are ready to uncle fuck ;)) */
    XStoreName(display, ROOT, PAWM_NAME);

    wm_adoptwins();
    SAY ("Initializing pimp...");
    pimp_init();
    SAY ("Initializing palabels...");
    palabel_init();
    SAY ("Initializing pabar...");
    pabar_init();
    
    tmpvalue=cfgpool_getstring(pawmpool,"paclock_module");
    if (strcmp(tmpvalue,"On")==0) {
        SAY ("Initializing paclock...");
        paclock_init();
    }
    free(tmpvalue);

    return !0;
}



/* Easy by now... */
void wm_done (void) {

    char *tmpvalue;

    if (--reference_count) return;

    SAY("Shutting down WM core");

    x_delsloot(ROOT, wm_handle, NULL);

    XSelectInput(display, ROOT, NoEventMask);
    
    tmpvalue=cfgpool_getstring(pawmpool,"paclock_module");
    if (strcmp(tmpvalue,"On")==0)
        paclock_done();
    free(tmpvalue);
 
    /* Shut properly all the classes */   
    pawindow_done();
    palabel_done();
    pabar_done();
    pimp_done();
    button_done();

    XStoreName(display, ROOT, "root");
    XSetInputFocus(display, ROOT, RevertToNone, CurrentTime);
}
/****************************************************************************/

==> pawm-2.2.2.tar.bz2: pawm-2.2.2/src/wm.c <==
/*
    Filename: wm.c
    Synopsis: Window management (general)

                          Copyright (C) 2000-2006
            Ral Nez de Arenas Coronado <raul@pleyades.net>
	         David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to:

        Free Software Foundation, Inc.
        59 Temple Place, Suite 330
        Boston, MA 02111-1307 USA

*/

#include "x.h"
#include "wm.h"
#include "pabar.h"
#include "pimp.h"
#include "hints.h"
#include "pawindow.h"
#include "keyboard.h"
#include "mobs.h"

#ifndef DISABLE_XRANDR
#include <X11/extensions/Xrandr.h>
static void wm_RRScreenChangeNotify (XEvent *);
#endif


/* Some definitions ********************************************************/
#define TOUGH  "Hey man, I'm the Puto Amo, and I want to nick my own sluts.\n"
#define POLITE "In other words, there is another Window Manager running...\n"
#define PAWM_EVMASK (SubstructureRedirectMask|ExposureMask|PropertyChangeMask)

/* Function prototypes *****************************************************/

/* Event handling */
static void wm_handle (XEvent *, void *);
static void wm_MapRequest       (XMapRequestEvent *);
static void wm_ConfigureRequest (XConfigureRequestEvent *);
static void wm_Expose		(XExposeEvent *);
static void wm_KeyPress		(XKeyPressedEvent *);

/* Private helper functions */
static void wm_adoptwins (void);
static int wm_tmpeh (Display *, XErrorEvent *);
/****************************************************************************/

static volatile sig_atomic_t FuckingBoss=1;  /* We are the Puto Amo */
static unsigned short int reference_count=0;

static struct {
    unsigned int current_desktop;
    unsigned int total_desktops;
    int randr_base;
} class;
    

/****************************************************************************/
/*                                                                          */
/*                        Event handling routines                          */
/*                                                                          */
/****************************************************************************/

static void wm_handle (XEvent *xevent, void *widget) {

    ASSERT(xevent);

    /* Widget is not need, just avoid the gcc warning */
    widget=NULL;

    switch (xevent->type) {
	case MapRequest:
		GOT(MapRequest, WM);
		wm_MapRequest(&(xevent->xmaprequest));
        break;
	case ConfigureRequest:
		GOT(ConfigureRequest, WM);
		wm_ConfigureRequest(&(xevent->xconfigurerequest));
        break;
	case Expose:
		GOT(Expose, WM);
		wm_Expose(&(xevent->xexpose));
	break;
	case ClientMessage:
		GOT(ClientMessage, WM);
		break;	
	case KeyPress:
		GOT(KeyPress, WM);
		wm_KeyPress(&(xevent->xkey));
		break;	
	case KeyRelease:
		GOT(KeyRelease, WM);	
		break;
	default:
#ifndef DISABLE_XRANDR
	if (xevent->type == class.randr_base ) {
		GOT(RRScreenChangeNotify, WM);
		wm_RRScreenChangeNotify(xevent);
	}
#else
		SAY("Event %d on WM from window %#lx ", xevent->type, xevent->xunmap.window);
#endif

    }
}




static void wm_MapRequest (XMapRequestEvent *event) {

    PAWindow pawindow=NULL;
    XWMHints *wmHints;
    Window root=None;
    Window parent=None;
    Window *winstack=NULL;
    unsigned int howmany=0;

    ASSERT(event);
    
    if (!XQueryTree(display, event->window, &root,
                    &parent, &winstack, &howmany)) return;
    if (winstack) XFree(winstack);

    /*
        Here we must deal with a race-condition. While a window is being
    handled it can unmap itself and after a little while remap. So we can
    have a MapRequest even that event should be directed to the 'handler'
    window. We try to get it solved here. Please note that grabbing the
    server doesn't solve this problem, since the event is put in the queue
    before we even think about doing the grab.
    */
    if (parent!=ROOT) {
        XEvent e;

        e.type=MapRequest;
        e.xmaprequest.serial=NextRequest(display);
        e.xmaprequest.send_event=True;
        e.xmaprequest.display=display;
        e.xmaprequest.parent=parent;
        e.xmaprequest.window=event->window;
                
        if (!XSendEvent(display, parent, False, SubstructureRedirectMask, &e))
            BUG();
        return;
    }

    /* Manage the fucking window */
    pawindow=pawindow_create(event->window);
    
    /* Get the initial_state property, if set by the client */
    wmHints=XGetWMHints(display, event->window);
    if ((wmHints) && (wmHints->flags & StateHint) && (wmHints->initial_state==IconicState)) {
		SAY("Client window wish to start iconified");
    } else {
		/* Client don't ask for IconicState..., map the fucking pawindow */
        //XGrabServer(display);
        pawindow_map(pawindow);
        //XUngrabServer(display);
    }
}




static void wm_ConfigureRequest (XConfigureRequestEvent *event) {

    XWindowChanges wc;
    Window root=None;
    Window parent=None;
    Window *winstack=NULL;
    unsigned int howmany=0;
    XEvent e;

    ASSERT(event);

    /*
        Here used to be a big BUG(), but since it faulted when a
    race condition arose, it was replaced by a shame return.
    X Clients really suck. GTK sucks. QT sucks, too. All sucks.
    */
    if (!XQueryTree(display, event->window, &root,
                    &parent, &winstack, &howmany)) return;
    if (winstack) XFree(winstack);

    if (parent!=ROOT) {
	
        e.type=ConfigureRequest;
        e.xconfigurerequest.serial=NextRequest(display);
        e.xconfigurerequest.send_event=True;
        e.xconfigurerequest.display=display;
        e.xconfigurerequest.parent=parent;
        e.xconfigurerequest.window=event->window;
        e.xconfigurerequest.x=event->x;
        e.xconfigurerequest.y=event->y;
        e.xconfigurerequest.width=event->width;
        e.xconfigurerequest.height=event->height;
        e.xconfigurerequest.border_width=event->border_width;
        e.xconfigurerequest.above=event->above;
        e.xconfigurerequest.detail=event->detail;
        e.xconfigurerequest.value_mask=event->value_mask;

        if (!XSendEvent(display, parent, False, SubstructureRedirectMask, &e))
            BUG();
	
	return;

    }

    /*
       As a guy told once upon a time: most of the fields
    contains crap, but there's no worry about that, since
    the X server (hopefully) won't take any members not
    specified in the value_mask. The Intel X server may
    be an exception, though. Intel...
    */
    wc.x=event->x;
    wc.y=event->y;
    wc.width=event->width;
    wc.height=event->height;
    //wc.border_width=event->border_width;
    wc.sibling=event->above;
    wc.stack_mode=event->detail;

    SAY("Window requesting to be configured to %d %d %d %d", event->width, event->height, event->x, event->y);
    XConfigureWindow(display, event->window, event->value_mask, &wc);
}


static void wm_Expose (XExposeEvent *event) {

    ASSERT(event);

    if (event->count) return;

}


#ifndef DISABLE_XRANDR
static void wm_RRScreenChangeNotify (XEvent *event) {
  
    ASSERT(event);

    XRRUpdateConfiguration(event);

    pabar_resize();
    /* TODO: Adjust icons to new size */
    //pabar_map();
}
#endif


static void wm_KeyPress (XKeyPressedEvent *event) {

    ASSERT(event);

	keyboard_gotkey(event->keycode, event->state);
	
}
/****************************************************************************/





/****************************************************************************/
/*                                                                          */
/*                      Private (static) functions                         */
/*                                                                          */
/****************************************************************************/

static int wm_tmpeh (Display *dpy, XErrorEvent *xerror) {

    ASSERT(dpy==display);

    dpy=NULL; xerror=NULL;

    /* Oh-oh... If we are here is because other WM is running */
    FuckingBoss=0;
    return 0;
}




/* This function adopts and manage all present windows on the desktop */
static void wm_adoptwins (void) {

    Window root=None;
    Window parent=None;
    Window *tree=NULL;
    unsigned int howmany=0;
    Status status;
    unsigned int i=0;

    /* Get info*/        
    status=XQueryTree(display, ROOT, &root, &parent, &tree, &howmany);

    /* Maybe no windows? */
    if ((!status) || (!howmany) || (!tree)) return;

    /* Try to manage all windows */
    for (i=0; i < howmany; i++) {
        XWindowAttributes attr;
        XEvent e;

        if(!XGetWindowAttributes(display, tree[i], &attr)) continue;
        /* Nothing to do if the window is not viewable */
        if (attr.map_state!=IsViewable) continue;
        if (attr.override_redirect==True) continue;

        XUnmapWindow(display, tree[i]);
        e.type=MapRequest;
        e.xmaprequest.serial=NextRequest(display);
        e.xmaprequest.send_event=False;
        e.xmaprequest.display=display;
        e.xmaprequest.parent=ROOT;
        e.xmaprequest.window=tree[i];

        /* Eat this... */
        if (!XSendEvent(display, ROOT, False, SubstructureRedirectMask, &e))
            BUG();
    }

    /* Be polite... */
    XFree(tree);
}
/****************************************************************************/




/****************************************************************************/
/*                                                                          */
/*                       Public (member) functions                         */
/*                                                                          */
/****************************************************************************/
int wm_init (void)
{
	Window fake;
	CARD32 tmpdata[20];
	unsigned long length;
	int (*olderrorhandler) (Display *, XErrorEvent *);

	SAY ("Initializing window manager...");

	if (reference_count++) return !0;

	olderrorhandler=XSetErrorHandler(wm_tmpeh);
	XSelectInput(display, ROOT, PAWM_EVMASK);
	XSync(display, False);
	XSetErrorHandler(olderrorhandler);

	if (!FuckingBoss) {
		fprintf(stderr, TOUGH);
		fprintf(stderr, POLITE);
		return 0;
	}

#ifndef DISABLE_XRANDR
	/* Check for XRandr extension */
	int major, minor,errbase;
	XRRQueryVersion(display, &major, &minor);
	XRRQueryExtension(display, &(class.randr_base), &errbase);
	SAY("Using XRandr %d.%d extension. Event base: %d Error base %d", major, minor, class.randr_base, errbase);
	XRRSelectInput(display, ROOT, RRScreenChangeNotifyMask);
#endif

	/* Only set the default solid background if there isn't a previous pixmap, check the _XROOTPMAP_ID atom */
	if (!x_get_property(ROOT, atoms.root_pixmap, XA_PIXMAP, &length))  {
		Pixmap root_pmap;
		GC gc;

		root_pmap=XCreatePixmap(display, ROOT, DWIDTH, DHEIGHT, DEPTH);	
		gc=XCreateGC(display, ROOT, 0, NULL);

		XSetForeground(display, gc, x_color("background"));
		XFillRectangle(display, root_pmap, gc, 0, 0, DWIDTH, DHEIGHT);
		XSetWindowBackgroundPixmap(display, ROOT, root_pmap);
		XClearWindow(display, ROOT);

		XFreeGC(display, gc);

		x_set_property(ROOT, atoms.root_pixmap, XA_PIXMAP, (unsigned char *)&root_pmap, 1);
	}
    
	/* Create fake window for _NET_SUPPORTING_WM_CHECK */
	fake=XCreateSimpleWindow(display, ROOT, 0, 0, 1, 1, 0, 0, BLACK);

	/* _NET_SUPPORTING_WM_CHECK */
	x_set_property(ROOT, atoms.net_supporting_wm_check, XA_WINDOW, (unsigned char *)&fake, 1);
	x_set_property(fake, atoms.net_supporting_wm_check, XA_WINDOW, (unsigned char *)&fake, 1);
	x_set_property_utf8(fake, atoms.net_wm_name, (unsigned char *)"pawm");

	/* _NET_SUPPORTED (Supported atoms) */
	tmpdata[0]=atoms.net_current_desktop;
	tmpdata[1]=atoms.net_active_window;
	tmpdata[2]=atoms.net_workarea;
	tmpdata[3]=atoms.net_wm_window_type;
	tmpdata[4]=atoms.net_wm_window_type_desktop;
	tmpdata[5]=atoms.net_wm_window_type_dock;
	tmpdata[6]=atoms.net_wm_window_type_toolbar;
	tmpdata[7]=atoms.net_wm_window_type_menu;
	tmpdata[8]=atoms.net_wm_window_type_utility;
	tmpdata[9]=atoms.net_wm_window_type_splash;
	tmpdata[10]=atoms.net_wm_window_type_dialog;
	tmpdata[11]=atoms.net_wm_window_type_normal;
	tmpdata[12]=atoms.net_wm_state;
	tmpdata[13]=atoms.net_wm_state_modal;
	tmpdata[14]=atoms.net_wm_state_sticky;
	tmpdata[15]=atoms.net_wm_state_maximized;
	tmpdata[16]=atoms.net_wm_state_shaded;
	tmpdata[17]=atoms.net_wm_state_skip_taskbar;
	tmpdata[18]=atoms.net_wm_state_skip_pager;
	tmpdata[19]=atoms.net_wm_state_hidden;
	tmpdata[20]=atoms.net_wm_state_fullscreen;
	tmpdata[21]=atoms.net_wm_state_above;
	tmpdata[22]=atoms.net_wm_state_below;
	tmpdata[23]=atoms.net_number_of_desktops;
	tmpdata[24]=atoms.net_client_list;
	x_set_property(ROOT, atoms.net_supported, XA_ATOM, (unsigned char *) tmpdata, 25);

	/* _NET_NUMBER_OF_DESKTOPS */
	x_set_property(ROOT, atoms.net_number_of_desktops, XA_CARDINAL, 
			   (unsigned char *)&(class.total_desktops), 1);

	/* _NET_CURRENT_DESKTOP - Always 0, pawm doesn't support virtual desktops */
	class.current_desktop=0;
	x_set_property(ROOT, atoms.net_current_desktop, XA_CARDINAL,
			   (unsigned char *)&(class.current_desktop), 1);

	keyboard_init();

	/* Initialize subsystems */
	SAY ("Initializing pawindows...");
	pawindow_init();

	/* Register the event handler ('sloot') */
	x_addsloot(ROOT, wm_handle, NULL);  /* No widget here */

	/* Signal API users that we are ready to uncle fuck ;)) */
	XStoreName(display, ROOT, PAWM_NAME);

	wm_adoptwins();
	SAY ("Initializing pimp...");
	pimp_init();
	SAY ("Initializing palabels...");
	palabel_init();
	SAY ("Initializing pabar...");
	pabar_init();

	return !0;
}



/* Easy by now... */
void wm_done (void) {

    //char *tmpvalue;

    if (--reference_count) return;

    SAY("Shutting down WM core");

    x_delsloot(ROOT, wm_handle, NULL);

    XSelectInput(display, ROOT, NoEventMask);
    
    /* Shut properly all the classes */   
    pawindow_done();
    palabel_done();
    pabar_done();
    pimp_done();
    button_done();

    XStoreName(display, ROOT, "root");
    XSetInputFocus(display, ROOT, RevertToNone, CurrentTime);
}
/****************************************************************************/
